// This file is part of the dune-gdt project:
//   https://github.com/dune-community/dune-gdt
// Copyright 2010-2016 dune-gdt developers and contributors. All rights reserved.
// License: BSD 2-Clause License (http://opensource.org/licenses/BSD-2-Clause)
// Authors:
//   Felix Schindler (2015 - 2016)

#include <dune/stuff/test/main.hxx> // <- this one has to come first

#include "operators/l2.hh"
#include "spaces/cg/pdelab.hh"
#include "spaces/dg/fem.hh"
#include "spaces/fv/default.hh"

using namespace Dune::GDT::Test;


#if HAVE_DUNE_FEM

typedef testing::Types<SPACE_DG_FEM_SGRID(1, 1, 2), SPACE_DG_FEM_SGRID(2, 1, 2), SPACE_DG_FEM_SGRID(3, 1, 2)>
    QuadraticSpaces;
TYPED_TEST_CASE(L2MatrixOperatorTest, QuadraticSpaces);

#elif HAVE_DUNE_PDELAB // HAVE_DUNE_FEM

typedef testing::Types<SPACE_CG_PDELAB_SGRID(1, 1, 1), SPACE_CG_PDELAB_SGRID(2, 1, 1), SPACE_CG_PDELAB_SGRID(3, 1, 1)>
    LinearSpaces;
TYPED_TEST_CASE(L2MatrixOperatorTest, LinearSpaces);

#else // HAVE_DUNE_FEM || HAVE_DUNE_PDELAB

typedef testing::Types<SPACE_FV_SGRID(1, 1), SPACE_FV_SGRID(2, 1), SPACE_FV_SGRID(3, 1)> ConstantSpaces;
TYPED_TEST_CASE(L2MatrixOperatorTest, ConstantSpaces);

#endif // HAVE_DUNE_FEM || HAVE_DUNE_PDELAB


TYPED_TEST(L2MatrixOperatorTest, constructible_by_ctor)
{
  this->constructible_by_ctor();
}
TYPED_TEST(L2MatrixOperatorTest, constructible_by_factory)
{
  this->constructible_by_factory();
}
TYPED_TEST(L2MatrixOperatorTest, is_matrix_operator)
{
  this->is_matrix_operator();
}
TYPED_TEST(L2MatrixOperatorTest, correct_for_constant_arguments)
{
  this->correct_for_constant_arguments();
}

#if HAVE_DUNE_FEM || HAVE_DUNE_PDELAB
TYPED_TEST(L2MatrixOperatorTest, correct_for_linear_arguments)
{
  this->correct_for_linear_arguments();
}
#else
TEST(DISABLED_L2MatrixOperatorTest, correct_for_linear_arguments)
{
  std::cerr << DSC::colorStringRed("Missing dependencies!") << std::endl;
}
#endif

#if HAVE_DUNE_FEM
TYPED_TEST(L2MatrixOperatorTest, correct_for_quadratic_arguments)
{
  this->correct_for_quadratic_arguments();
}
#else
TEST(DISABLED_L2MatrixOperatorTest, correct_for_quadratic_arguments)
{
  std::cerr << DSC::colorStringRed("Missing dependencies!") << std::endl;
}
#endif
